//-----------------------------------------------------------------------------------
// configuration settings
//-----------------------------------------------------------------------------------

// PCB type
#define PCB_SMALL 0      // 1.25"
#define PCB_MED   1      // 2.00"
#define PCB_TYPE PCB_SMALL

// LED array (brightness)
#define LED_TYPE_KB 0    // 1'st batch from Sparkfun (Kingbright 1288; nice and bright)
#define LED_TYPE_YS 1    // 2'nd batch from Sparkfun (Young Sun;  dismally dim)
#define LED_TYPE LED_TYPE_YS

// photography demo display
#define ENABLE_DEMO_DISPLAY

//-----------------------------------------------------------------------------------
// chip selection
//
// NOTE - THESE VALUES MUST MATCH "device.inc"
//-----------------------------------------------------------------------------------

#include "p33FJ64GP802.h"

//-- installed crystal (only 4,8,10 MHz supported)

#if PCB_TYPE == PCB_MED
#define XTAL_MHZ  4 // min  4 MHz
#endif

#if PCB_TYPE == PCB_SMALL
#define XTAL_MHZ 10 // max 10 MHz
#endif

//-- CPU cycle speed (only 10,20,40 MIPS supported)

#define CPU_MIPS 10 // == fOsc/2 == min practical for this application
//efine CPU_MIPS 20 // == fOsc/2
//efine CPU_MIPS 40 // == fOsc/2 == max possible for this CPU chip

//-----------------------------------------------------------------------------------
// I/O ports
//-----------------------------------------------------------------------------------

// A/D channels
#define ADCH_L    0     // audio left channel
#define ADCH_R    1     // audio right channel
#define ADCH_PHOT 5     // photocell

// EE (contained in digital pot chip)
#define POT_CMD_READ  0x0C00
#define POT_CMD_WRITE 0x0000
#define POT_WIPER_L  (0x00<<12) // left channel gain
#define POT_WIPER_R  (0x01<<12) // right channel gain
#define POT_EE_WIPER (0x02<<12) // nonvolatile gain setting (both channels)
#define POT_EE_USER  (0x06<<12) // "mode" storage location

// remappable input sources [see dsPIC33FJ64GP802 datasheet]
#define RPIN_SDI    8
// remappable output pins [see dsPIC33FJ64GP802 datasheet]
#define RPOR_TXD1   0x03
#define RPOR_SDO1   0x07
#define RPOR_SCK1   0x08
#define RPOR_OC1    0x12

// I/O pin control macros

#define RCK_OFF       {__asm__ volatile ("bclr LATB,#7");}
#define RCK_ON        {__asm__ volatile ("bset LATB,#7");}
#define CLR_OFF       {__asm__ volatile ("bset LATB,#10");}
#define CLR_ON        {__asm__ volatile ("bclr LATB,#10");}

#if PCB_TYPE == PCB_SMALL
// original PCB layout
#define CS_DIGPOT_OFF {__asm__ volatile ("bset LATA,#4");}
#define CS_DIGPOT_ON  {__asm__ volatile ("bclr LATA,#4");}
#define CS_MEM_OFF    {__asm__ volatile ("bset LATB,#10");}
#define CS_MEM_ON     {__asm__ volatile ("bclr LATB,#10");}
#define LED_DIAG_OFF  {__asm__ volatile ("bset LATB,#2");}
#define LED_DIAG_ON   {__asm__ volatile ("bclr LATB,#2");}
#define LED_CLIP_OFF  {__asm__ volatile ("bset LATB,#12");}
#define LED_CLIP_ON   {__asm__ volatile ("bclr LATB,#12");}
#endif

#if PCB_TYPE == PCB_MED
// RA4 is now DIAG
// RB2 is now GAIN
// RB10 is now CLIP
// MEM is gone
// RB12(AN12) is now OPT [an output for now]
#define CS_DIGPOT_OFF {__asm__ volatile ("bset LATB,#2");}
#define CS_DIGPOT_ON  {__asm__ volatile ("bclr LATB,#2");}
#define LED_DIAG_OFF  {__asm__ volatile ("bset LATA,#4");}
#define LED_DIAG_ON   {__asm__ volatile ("bclr LATA,#4");}
#define LED_CLIP_OFF  {__asm__ volatile ("bset LATB,#10");}
#define LED_CLIP_ON   {__asm__ volatile ("bclr LATB,#10");}
#define CS_MEM_OFF    // not implemented on this PCB type
#endif

// I/O input macros
#define BTN_MODE PORTBbits.RB15
#define BTN_UP   PORTBbits.RB6
#define BTN_DOWN PORTBbits.RB5

// error codes - diagnostic LED blinks to indicate which error occurred
#define E_WDT       1
#define E_CONFIG    2
#define E_OPCODE    3
#define E_TRAP      4
#define E_SPI       5
#define E_MUTEX     6
#define E_OSCFAIL   7
#define E_ADDRESS   8
#define E_STACK     9
#define E_MATH     10
#define E_DMA      11
