/*
    Error handlers for Spectrum Analyzer application
*/

#include "sa.h"

extern void Delay_Code_msec( unsigned int msecs );

void fatal_error_handler(unsigned int errorNumber)
{
    SRbits.IPL = 7;       // disable interrupts
	RCONbits.SWDTEN = 0;  // disable watchdog

    CLR_ON                // turn off all LED row drivers
    CS_DIGPOT_OFF         // turn off digital pot chip select
    CS_MEM_OFF            // turn off memory chip select

    OC1RS = 2047;         // turn off PWM output
	AD1CON1bits.ADON = 0; // shut down ADC
    DMA0CONbits.CHEN = 0; // shut down DMA
    T1CONbits.TON = 0;    // turn off all timers
    T2CONbits.TON = 0;
    T3CONbits.TON = 0;

    while (1) // blink error number on diagnostic LED
    {
        unsigned int n;

        for (n = 0; n < errorNumber; n++)
        {
            LED_DIAG_ON
            Delay_Code_msec( 200 );
            LED_DIAG_OFF
            Delay_Code_msec( 200 );
        }
        Delay_Code_msec( 1500 );
    }
}

/*
   Primary Exception Vector handlers:
   These routines are used if INTCON2bits.ALTIVT = 0.
*/

void __attribute__((interrupt, no_auto_psv)) _OscillatorFail(void)
{
    fatal_error_handler(E_OSCFAIL);
}

void __attribute__((interrupt, no_auto_psv)) _AddressError(void)
{
    fatal_error_handler(E_ADDRESS);
}

void __attribute__((interrupt, no_auto_psv)) _StackError(void)
{
    fatal_error_handler(E_STACK);
}

void __attribute__((interrupt, no_auto_psv)) _MathError(void)
{
    fatal_error_handler(E_MATH);
}

void __attribute__((interrupt, no_auto_psv)) _DMACError(void)
{
    fatal_error_handler(E_DMA);
}


/* 
   Alternate Exception Vector handlers:
   These routines are used if INTCON2bits.ALTIVT = 1.
*/

void __attribute__((interrupt, no_auto_psv)) _AltOscillatorFail(void)
{
    fatal_error_handler(E_OSCFAIL);
}

void __attribute__((interrupt, no_auto_psv)) _AltAddressError(void)
{
    fatal_error_handler(E_ADDRESS);
}

void __attribute__((interrupt, no_auto_psv)) _AltStackError(void)
{
    fatal_error_handler(E_STACK);
}

void __attribute__((interrupt, no_auto_psv)) _AltMathError(void)
{
    fatal_error_handler(E_MATH);
}

void __attribute__((interrupt, no_auto_psv)) _AltDMACError(void)
{
    fatal_error_handler(E_DMA);
}
