;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

    .include "sa.inc" ; hardware registers for particular CPU chip being used

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	.section .libdsp, code

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Multiply a data array by the windowing function
;
; Input:
;   w0 = pointer to data (input and output)
;   hamming_window_ram = window function constants (1'st half only)
;
; System resources usage:
;	{w0..w7}	used, not restored
;
; DO and REPEAT instruction usage.
;	1 level DO intruction
;	no REPEAT intructions
;
;............................................................................

 .global _Window_2048;

_Window_2048:

;............................................................................

	; Prepare core for fractional computation
    ; + enable saturation on both accumulators
    ; + enable data space write saturation
    ; + normal saturation (1:31)
    ; + unbiased rounding
    ; + fractional multiply mode
    push	CORCON
    mov     #((1<<SATA)|(1<<SATB)|(1<<SATDW)), w7
    mov     w7, CORCON

;............................................................................

    mov     #(1024-1), w3 ; iterations per half (for 2048 input samples)

;-- 1'st half

    mov     #_hamming_window_ram, w1

    do      w3, _ENDLOOP1
    ; apply window function
    mov     [w0],   w4  ; get source value
    mov     [w1++], w5  ; get window factor
    mpy     w4*w5, a    ; multiply src * factor
_ENDLOOP1: ; loop execution ends at the FOLLOWING INSTRUCTION...
    sac.r   a,[w0++]    ; store result

;-- 2'nd half

    do      w3, _ENDLOOP2
    ; apply window function
    mov     [w0],   w4  ; get source value
    mov     [--w1], w5  ; get window factor
    mpy     w4*w5, a    ; multiply src * factor
_ENDLOOP2: ; loop execution ends at the FOLLOWING INSTRUCTION...
    sac.r   a,[w0++]    ; store result

;............................................................................

	pop	CORCON

;............................................................................

    return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	.end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
