library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

package am2901_mnemonics is

--
-- Am2901 Source Operands Control
--
  constant AQ : std_logic_vector(2 downto 0) := "000";
  constant AB : std_logic_vector(2 downto 0) := "001";
  constant ZQ : std_logic_vector(2 downto 0) := "010";
  constant ZB : std_logic_vector(2 downto 0) := "011";
  constant ZA : std_logic_vector(2 downto 0) := "100";
  constant DA : std_logic_vector(2 downto 0) := "101";
  constant DQ : std_logic_vector(2 downto 0) := "110";
  constant DZ : std_logic_vector(2 downto 0) := "111";

--
-- Am2901 ALU Function Control
--
  constant ALU_ADD   : std_logic_vector(2 downto 0) := "000";  -- R + S
  constant ALU_SUBR  : std_logic_vector(2 downto 0) := "001";  -- S - R
  constant ALU_SUBS  : std_logic_vector(2 downto 0) := "010";  -- R - S
  constant ALU_OR    : std_logic_vector(2 downto 0) := "011";  -- R OR S
  constant ALU_AND   : std_logic_vector(2 downto 0) := "100";  -- R AND S
  constant ALU_NOTRS : std_logic_vector(2 downto 0) := "101";  -- NOT R OR S
  constant ALU_EXOR  : std_logic_vector(2 downto 0) := "110";  -- R XOR S
  constant ALU_EXNOR : std_logic_vector(2 downto 0) := "111";  -- NOT (R XOR S)

--
-- Am2901 Destination Control
--
  constant QREG  : std_logic_vector(2 downto 0) := "000";  -- None     F -> Q
  constant NOP   : std_logic_vector(2 downto 0) := "001";  -- None     None
  constant RAMA  : std_logic_vector(2 downto 0) := "010";  -- F -> B   None
  constant RAMF  : std_logic_vector(2 downto 0) := "011";  -- F -> B   Y = F
  constant RAMQD : std_logic_vector(2 downto 0) := "100";  -- F/2 -> B Q/2 -> Q
  constant RAMD  : std_logic_vector(2 downto 0) := "101";  -- F/2 ->   B None
  constant RAMQU : std_logic_vector(2 downto 0) := "110";  -- 2F -> B  2Q -> Q
  constant RAMU  : std_logic_vector(2 downto 0) := "111";  -- 2F -> B  None

--
-- Am2901 RAM Register Control
--
  constant RAM_HOLD : std_logic := '0';  -- Do not write to RAM registers
  constant RAM_LOAD : std_logic := '1';  -- Write to RAM registers

end am2901_mnemonics;
