library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

use work.am2901_comp.all;
use work.am2901_mnemonics.all;

entity instruction_decode is

  port (
    I_in                  : in  std_logic_vector(8 downto 0);
    RAM_WE_out            : out std_logic;
    RAM_Shift_Cntrl_out   : out std_logic_vector(2 downto 0);
    Q_Shift_Cntrl_out     : out std_logic_vector(2 downto 0);
    ALU_Cntrl_out         : out std_logic_vector(2 downto 0);
    ALU_Source_Select_out : out std_logic_vector(2 downto 0);
    Output_Select_out     : out std_logic_vector(2 downto 0));

end instruction_decode;

architecture arch_instruction_decode of instruction_decode is

  signal I                 : std_logic_vector(8 downto 0);
  signal RAM_WE            : std_logic;
  signal RAM_Shift_Cntrl   : std_logic_vector(2 downto 0);
  signal Q_Shift_Cntrl     : std_logic_vector(2 downto 0);
  signal ALU_Cntrl         : std_logic_vector(2 downto 0);
  signal ALU_Source_Select : std_logic_vector(2 downto 0);
  signal Output_Select     : std_logic_vector(2 downto 0);

  alias ALU_Source      : std_logic_vector(2 downto 0) is I(2 downto 0);
  alias ALU_Function    : std_logic_vector(2 downto 0) is I(5 downto 3);
  alias ALU_Destination : std_logic_vector(2 downto 0) is I(8 downto 6);

begin  -- arch_instruction_decode

  I <= I_in;

  RAM_WE_out            <= RAM_WE;
  RAM_Shift_Cntrl_out   <= RAM_Shift_Cntrl;
  Q_Shift_Cntrl_out     <= Q_Shift_Cntrl;
  ALU_Cntrl_out         <= ALU_Cntrl;
  ALU_Source_Select_out <= ALU_Source_Select;
  Output_Select_out     <= Output_Select;

  RAM_Shift_Cntrl   <= ALU_Destination;
  Q_Shift_Cntrl     <= ALU_Destination;
  ALU_Cntrl         <= ALU_Function;
  ALU_Source_Select <= ALU_Source;
  Output_Select     <= ALU_Destination;

  U1 : process (ALU_Destination)
  begin  -- process U1

    case ALU_Destination is

      when QREG =>
        RAM_WE <= RAM_HOLD;

      when NOP =>
        RAM_WE <= RAM_HOLD;

      when RAMA =>
        RAM_WE <= RAM_LOAD;

      when RAMF =>
        RAM_WE <= RAM_LOAD;

      when RAMQD =>
        RAM_WE <= RAM_LOAD;

      when RAMD =>
        RAM_WE <= RAM_LOAD;

      when RAMQU =>
        RAM_WE <= RAM_LOAD;

      when RAMU =>
        RAM_WE <= RAM_LOAD;

      when others => null;

    end case;  -- case ALU_Destination

  end process U1;

end arch_instruction_decode;
