library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

use work.am2901_comp.all;
use work.am2901_mnemonics.all;

entity output_mux is

  generic (
    WIDTH            :     integer := 4);
  port (
    A_Data_in        : in  std_logic_vector((WIDTH - 1) downto 0);
    F_in             : in  std_logic_vector((WIDTH - 1) downto 0);
    Output_Select_in : in  std_logic_vector(2 downto 0);
    OE_in            : in  std_logic;
    Y_out            : out std_logic_vector((WIDTH - 1) downto 0));

end output_mux;

architecture arch_output_mux of output_mux is

  signal A_Data        : std_logic_vector((WIDTH - 1) downto 0);
  signal F             : std_logic_vector((WIDTH - 1) downto 0);
  signal Output_Select : std_logic_vector(2 downto 0);
  signal OE            : std_logic;
  signal Y             : std_logic_vector((WIDTH - 1) downto 0);

begin  -- arch_output_mux

  A_Data        <= A_Data_in;
  F             <= F_in;
  Output_Select <= Output_Select_in;
  OE            <= OE_in;

  U1 : process (A_Data, F, Output_Select)
  begin  -- process U1

    case Output_Select is
      when QREG =>
        Y <= F;

      when NOP =>
        Y <= F;

      when RAMA =>
        Y <= A_Data;

      when RAMF =>
        Y <= F;

      when RAMQD =>
        Y <= F;

      when RAMD =>
        Y <= F;

      when RAMQU =>
        Y <= F;

      when RAMU =>
        Y <= F;

      when others => null;

    end case;  -- case Output_Select

  end process U1;

  U2 : process (OE, Y)
  begin  -- process U2

    if OE = '1' then

      Y_out <= Y;

    else

      Y_out <= (others => '0');

    end if;  -- if OE = '1'

  end process U2;
  
end arch_output_mux;
