library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

use work.am2901_comp.all;
use work.am2901_mnemonics.all;

entity q_shift is

  generic (
    WIDTH : integer := 4);

  port (
    Q_Shift_in       : in  std_logic_vector((WIDTH - 1) downto 0);
    F_in             : in  std_logic_vector((WIDTH - 1) downto 0);
    Q_Shift_Cntrl_in : in  std_logic_vector(2 downto 0);
    Q_LSB_in         : in  std_logic;
    Q_MSB_in         : in  std_logic;
    Q_LSB_out        : out std_logic;
    Q_MSB_out        : out std_logic;
    Q_LSB_Dir_out    : out std_logic;
    Q_MSB_Dir_out    : out std_logic;
    Q_Shift_out      : out std_logic_vector((WIDTH - 1) downto 0));
end q_shift;

architecture arch_q_shift of q_shift is

  signal F             : std_logic_vector((WIDTH - 1) downto 0);
  signal Q_Shift_Cntrl : std_logic_vector(2 downto 0);
  signal Q_LSB_Dir     : std_logic;
  signal Q_MSB_Dir     : std_logic;

begin  -- arch_q_shift

  F             <= F_in;
  Q_Shift_Cntrl <= Q_Shift_Cntrl_in;

  Q_LSB_Dir_out <= Q_LSB_Dir;
  Q_MSB_Dir_out <= Q_MSB_Dir;

  -- Q_xxx_Dir = '1' direction is input
  -- Q_xxx_Dir = '0' direction is output
  -- xxx = LSB or MSB

  U1 : process (Q_Shift_in, F, Q_Shift_Cntrl, Q_LSB_in, Q_MSB_in)
  begin  -- process U1

    case Q_Shift_Cntrl is

      when QREG =>
        Q_Shift_out <= F;

        Q_LSB_out <= '0';
        Q_LSB_Dir <= '1';

        Q_MSB_out <= '0';
        Q_MSB_Dir <= '1';

      when NOP =>
        Q_Shift_out <= Q_Shift_in;

        Q_LSB_out <= '0';
        Q_LSB_Dir <= '1';

        Q_MSB_out <= '0';
        Q_MSB_Dir <= '1';

      when RAMA =>
        Q_Shift_out <= Q_Shift_in;

        Q_LSB_out <= '0';
        Q_LSB_Dir <= '1';

        Q_MSB_out <= '0';
        Q_MSB_Dir <= '1';

      when RAMF =>
        Q_Shift_out <= Q_Shift_in;

        Q_LSB_out <= '0';
        Q_LSB_Dir <= '1';

        Q_MSB_out <= '0';
        Q_MSB_Dir <= '1';

      when RAMQD =>
        Q_Shift_out(WIDTH - 1) <= Q_MSB_in;
        for ramqd_index in (WIDTH - 2) downto 0 loop

          Q_Shift_out(ramqd_index) <= Q_Shift_in(ramqd_index + 1);

        end loop;  -- ramqd_index

        Q_LSB_out <= Q_Shift_in(0);
        Q_LSB_Dir <= '0';

        Q_MSB_out <= '0';
        Q_MSB_Dir <= '1';

      when RAMD =>
        Q_Shift_out <= Q_Shift_in;

        Q_LSB_out <= Q_Shift_in(0);
        Q_LSB_Dir <= '0';

        Q_MSB_out <= '0';
        Q_MSB_Dir <= '1';

      when RAMQU =>
        for ramqu_index in (WIDTH - 1) downto 1 loop

          Q_Shift_out(ramqu_index) <= Q_Shift_in(ramqu_index - 1);

        end loop;  -- ramqu_index
        Q_Shift_out(0) <= Q_LSB_in;

        Q_LSB_out <= '0';
        Q_LSB_Dir <= '1';

        Q_MSB_out <= Q_Shift_in(WIDTH - 1);
        Q_MSB_Dir <= '0';

      when RAMU =>
        Q_Shift_out <= Q_Shift_in;

        Q_LSB_out <= '0';
        Q_LSB_Dir <= '1';

        Q_MSB_out <= Q_Shift_in(WIDTH - 1);
        Q_MSB_Dir <= '0';

      when others => null;

    end case;

  end process U1;

end arch_q_shift;
