library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

library unisim;
use unisim.all;

entity reg_ram is

  generic (
    WIDTH : integer := 4);

  port (
    Clock_in     : in  std_logic;
    Data_in      : in  std_logic_vector((WIDTH - 1) downto 0);
    A_Address_in : in  std_logic_vector(3 downto 0);
    B_Address_in : in  std_logic_vector(3 downto 0);
    RAM_WE_in    : in  std_logic;
    A_Data_out   : out std_logic_vector((WIDTH - 1) downto 0);
    B_Data_out   : out std_logic_vector((WIDTH - 1) downto 0));

end reg_ram;

architecture arch_reg_ram of reg_ram is

  component RAM16X1D
    generic (
      INIT  :     bit_vector);
    port (
      DPO   : out std_ulogic;
      SPO   : out std_ulogic;
      A0    : in  std_ulogic;
      A1    : in  std_ulogic;
      A2    : in  std_ulogic;
      A3    : in  std_ulogic;
      D     : in  std_ulogic;
      DPRA0 : in  std_ulogic;
      DPRA1 : in  std_ulogic;
      DPRA2 : in  std_ulogic;
      DPRA3 : in  std_ulogic;
      WCLK  : in  std_ulogic;
      WE    : in  std_ulogic);
  end component;

  attribute box_type             : string;
  attribute box_type of RAM16X1D : component is "black_box";

  signal Clock     : std_logic;
  signal Data      : std_logic_vector((WIDTH - 1) downto 0);
  signal A_Address : std_logic_vector(3 downto 0);
  signal B_Address : std_logic_vector(3 downto 0);
  signal RAM_WE    : std_logic;
  signal A_Data    : std_logic_vector((WIDTH - 1) downto 0);
  signal B_Data    : std_logic_vector((WIDTH - 1) downto 0);

begin  -- arch_reg_ram

  Clock     <= Clock_in;
  Data      <= Data_in;
  A_Address <= A_Address_in;
  B_Address <= B_Address_in;
  RAM_WE    <= RAM_WE_in;

  A_Data_out <= A_Data;
  B_Data_out <= B_Data;

  RAM_Loop : for ram_index in (WIDTH - 1) downto 0 generate

    U0 : RAM16X1D
      generic map (
        INIT   => X"0000")
      port map (
        DPO   => A_Data(ram_index),
        SPO   => B_Data(ram_index),
        A0    => B_Address(0),
        A1    => B_Address(1),
        A2    => B_Address(2),
        A3    => B_Address(3),
        D     => Data(ram_index),
        DPRA0 => A_Address(0),
        DPRA1 => A_Address(1),
        DPRA2 => A_Address(2),
        DPRA3 => A_Address(3),
        WCLK  => Clock,
        WE    => RAM_WE);

  end generate RAM_Loop;

end arch_reg_ram;
