library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;

use work.am2910_comp.all;
use work.am2910_const.all;

entity instruction_pla is
  port (
    Cond_Code_in    : in  std_logic;
    Cond_Code_En_in : in  std_logic;
    Instr_in        : in  std_logic_vector(3 downto 0);
    RegZero_in      : in  std_logic;
    Mux_Sel_out     : out std_logic_vector(1 downto 0);
    Mux_En_out      : out std_logic;
    Reg_Decr_out    : out std_logic;
    Reg_Load_out    : out std_logic;
    Stk_Clr_out     : out std_logic;
    Stk_Push_out    : out std_logic;
    Stk_Pop_out     : out std_logic;
    PL_En_out       : out std_logic;
    Map_En_out      : out std_logic;
    Vect_En_out     : out std_logic);
end instruction_pla;

architecture arch_instruction_pla of instruction_pla is

  signal Cond_Code    : std_logic;
  signal Cond_Code_En : std_logic;
  signal Instr        : std_logic_vector(3 downto 0);
  signal RegZero      : std_logic;
  signal PL_En        : std_logic;
  signal Map_En       : std_logic;
  signal Vect_En      : std_logic;

  signal Stk_Cntrl : std_logic_vector(2 downto 0);
  signal Reg_Cntrl : std_logic_vector(1 downto 0);
  signal Mux_Cntrl : std_logic_vector(2 downto 0);

begin

  Cond_Code    <= Cond_Code_in;
  Cond_Code_En <= Cond_Code_En_in;
  Instr        <= Instr_in;
  RegZero      <= RegZero_in;

  Stk_Clr_out  <= Stk_Cntrl(2);
  Stk_Push_out <= Stk_Cntrl(1);
  Stk_Pop_out  <= Stk_Cntrl(0);

  Reg_Decr_out <= Reg_Cntrl(1);
  Reg_Load_out <= Reg_Cntrl(0);

  Mux_Sel_out(0) <= Mux_Cntrl(0);
  Mux_Sel_out(1) <= Mux_Cntrl(1);
  Mux_En_out     <= Mux_Cntrl(2);

  PL_En_out   <= PL_En;
  Map_En_out  <= Map_En;
  Vect_En_out <= Vect_En;

  U1 : process (Instr, Cond_Code, Cond_Code_En, RegZero)

  begin
    case Instr is

      when uinst_JZ =>

        Mux_Cntrl <= mux_ZERO;
        Reg_Cntrl <= register_HOLD;
        Stk_Cntrl <= stack_CLEAR;
        PL_En     <= pl_ENABLE;
        Map_En    <= map_DISABLE;
        Vect_En   <= vect_DISABLE;

      when uinst_CJS =>

        if ((Cond_Code_En = '0') or (Cond_Code = '1')) then

          Mux_Cntrl <= mux_DIRECT;
          Reg_Cntrl <= register_HOLD;
          Stk_Cntrl <= stack_PUSH;
          PL_En     <= pl_ENABLE;
          Map_En    <= map_DISABLE;
          Vect_En   <= vect_DISABLE;

        else

          Mux_Cntrl <= mux_uPC;
          Reg_Cntrl <= register_HOLD;
          Stk_Cntrl <= stack_HOLD;
          PL_En     <= pl_ENABLE;
          Map_En    <= map_DISABLE;
          Vect_En   <= vect_DISABLE;

        end if;  --if ((Cond_Code_En = '0') or (Cond_Code = '1'))

      when uinst_JMAP =>

        Mux_Cntrl <= mux_DIRECT;
        Reg_Cntrl <= register_HOLD;
        Stk_Cntrl <= stack_HOLD;
        PL_En     <= pl_DISABLE;
        Map_En    <= map_ENABLE;
        Vect_En   <= vect_DISABLE;

      when uinst_CJP =>

        if ((Cond_Code_En = '0') or (Cond_Code = '1')) then

          Mux_Cntrl <= mux_DIRECT;
          Reg_Cntrl <= register_HOLD;
          Stk_Cntrl <= stack_HOLD;
          PL_En     <= pl_ENABLE;
          Map_En    <= map_DISABLE;
          Vect_En   <= vect_DISABLE;

        else

          Mux_Cntrl <= mux_uPC;
          Reg_Cntrl <= register_HOLD;
          Stk_Cntrl <= stack_HOLD;
          PL_En     <= pl_ENABLE;
          Map_En    <= map_DISABLE;
          Vect_En   <= vect_DISABLE;

        end if;  --if ((Cond_Code_En = '0') or (Cond_Code = '1'))

      when uinst_PUSH =>

        if ((Cond_Code_En = '1') and (Cond_Code = '0')) then

          Mux_Cntrl <= mux_uPC;
          Reg_Cntrl <= register_HOLD;
          Stk_Cntrl <= stack_PUSH;
          PL_En     <= pl_ENABLE;
          Map_En    <= map_DISABLE;
          Vect_En   <= vect_DISABLE;

        else

          Mux_Cntrl <= mux_uPC;
          Reg_Cntrl <= register_LOAD;
          Stk_Cntrl <= stack_PUSH;
          PL_En     <= pl_ENABLE;
          Map_En    <= map_DISABLE;
          Vect_En   <= vect_DISABLE;

        end if;  --if ((Cond_Code_En = '1') and (Cond_Code = '0'))

      when uinst_JSRP =>

        if ((Cond_Code_En = '0') or (Cond_Code = '1')) then

          Mux_Cntrl <= mux_DIRECT;
          Reg_Cntrl <= register_HOLD;
          Stk_Cntrl <= stack_PUSH;
          PL_En     <= pl_ENABLE;
          Map_En    <= map_DISABLE;
          Vect_En   <= vect_DISABLE;

        else

          Mux_Cntrl <= mux_REG;
          Reg_Cntrl <= register_HOLD;
          Stk_Cntrl <= stack_PUSH;
          PL_En     <= pl_ENABLE;
          Map_En    <= map_DISABLE;
          Vect_En   <= vect_DISABLE;

        end if;  --if ((Cond_Code_En = '0') or (Cond_Code = '1'))

      when uinst_CJV =>

        if ((Cond_Code_En = '0') or (Cond_Code = '1')) then

          Mux_Cntrl <= mux_DIRECT;
          Reg_Cntrl <= register_HOLD;
          Stk_Cntrl <= stack_HOLD;
          PL_En     <= pl_DISABLE;
          Map_En    <= map_DISABLE;
          Vect_En   <= vect_ENABLE;

        else

          Mux_Cntrl <= mux_uPC;
          Reg_Cntrl <= register_HOLD;
          Stk_Cntrl <= stack_HOLD;
          PL_en     <= pl_DISABLE;
          Map_En    <= map_DISABLE;
          vect_en   <= vect_ENABLE;

        end if;  --if ((Cond_Code_En = '0') or (Cond_Code = '1'))

      when uinst_JRP =>

        if ((Cond_Code_En = '0') or (Cond_Code = '1')) then

          Mux_Cntrl <= mux_DIRECT;
          Reg_Cntrl <= register_HOLD;
          Stk_Cntrl <= stack_HOLD;
          PL_En     <= pl_ENABLE;
          Map_En    <= map_DISABLE;
          Vect_En   <= vect_DISABLE;

        else

          Mux_Cntrl <= mux_REG;
          Reg_Cntrl <= register_HOLD;
          Stk_Cntrl <= stack_HOLD;
          PL_En     <= pl_ENABLE;
          Map_En    <= map_DISABLE;
          Vect_En   <= vect_DISABLE;

        end if;  --if ((Cond_Code_En = '0') or (Cond_Code = '1'))

      when uinst_RFCT =>

        if ((Cond_Code_En = '0') or (Cond_Code = '1')) then

          if RegZero = REGeqZERO then

            Mux_Cntrl <= mux_uPC;
            Reg_Cntrl <= register_HOLD;
            Stk_Cntrl <= stack_POP;
            PL_En     <= pl_ENABLE;
            Map_En    <= map_DISABLE;
            Vect_En   <= vect_DISABLE;

          else

            Mux_Cntrl <= mux_STACK;
            Reg_Cntrl <= register_DECR;
            Stk_Cntrl <= stack_HOLD;
            PL_En     <= pl_ENABLE;
            Map_En    <= map_DISABLE;
            Vect_En   <= vect_DISABLE;

          end if;  -- if RegZero = REGeqZERO 

        else

          if RegZero = REGeqZERO then

            Mux_Cntrl <= mux_uPC;
            Reg_Cntrl <= register_HOLD;
            Stk_Cntrl <= stack_POP;
            PL_En     <= pl_ENABLE;
            Map_En    <= map_DISABLE;
            Vect_En   <= vect_DISABLE;

          else

            Mux_Cntrl <= mux_STACK;
            Reg_Cntrl <= register_DECR;
            Stk_Cntrl <= stack_HOLD;
            PL_En     <= pl_ENABLE;
            Map_En    <= map_DISABLE;
            Vect_En   <= vect_DISABLE;

          end if;  -- if RegZero = REGeqZERO 

        end if;  --if ((Cond_Code_En = '0') or (Cond_Code = '1'))

      when uinst_RPCT =>

        if ((Cond_Code_En = '0') or (Cond_Code = '1')) then

          if RegZero = REGeqZERO then

            Mux_Cntrl <= mux_uPC;
            Reg_Cntrl <= register_HOLD;
            Stk_Cntrl <= stack_HOLD;
            PL_En     <= pl_ENABLE;
            Map_En    <= map_DISABLE;
            Vect_En   <= vect_DISABLE;

          else

            Mux_Cntrl <= mux_DIRECT;
            Reg_Cntrl <= register_DECR;
            Stk_Cntrl <= stack_HOLD;
            PL_En     <= pl_ENABLE;
            Map_En    <= map_DISABLE;
            Vect_En   <= vect_DISABLE;

          end if;  -- if RegZero = REGeqZERO

        else

          if RegZero = REGeqZERO then

            Mux_Cntrl <= mux_uPC;
            Reg_Cntrl <= register_HOLD;
            Stk_Cntrl <= stack_HOLD;
            PL_En     <= pl_ENABLE;
            Map_En    <= map_DISABLE;
            Vect_En   <= vect_DISABLE;

          else

            Mux_Cntrl <= mux_DIRECT;
            Reg_Cntrl <= register_DECR;
            Stk_Cntrl <= stack_HOLD;
            PL_En     <= pl_ENABLE;
            Map_En    <= map_DISABLE;
            Vect_En   <= vect_DISABLE;

          end if;  -- if RegZero = REGeqZERO

        end if;  --if ((Cond_Code_En = '0') or (Cond_Code = '1'))

      when uinst_CRTN =>

        if ((Cond_Code_En = '0') or (Cond_Code = '1')) then

          Mux_Cntrl <= mux_STACK;
          Reg_Cntrl <= register_HOLD;
          Stk_Cntrl <= stack_POP;
          PL_En     <= pl_ENABLE;
          Map_En    <= map_DISABLE;
          Vect_En   <= vect_DISABLE;

        else

          Mux_Cntrl <= mux_uPC;
          Reg_Cntrl <= register_HOLD;
          Stk_Cntrl <= stack_HOLD;
          PL_En     <= pl_ENABLE;
          Map_En    <= map_DISABLE;
          Vect_En   <= vect_DISABLE;

        end if;  --if ((Cond_Code_En = '0') or (Cond_Code = '1'))

      when uinst_CJPP =>

        if ((Cond_Code_En = '0') or (Cond_Code = '1')) then

          Mux_Cntrl <= mux_DIRECT;
          Reg_Cntrl <= register_HOLD;
          Stk_Cntrl <= stack_POP;
          PL_En     <= pl_ENABLE;
          Map_En    <= map_DISABLE;
          Vect_En   <= vect_DISABLE;

        else

          Mux_Cntrl <= mux_uPC;
          Reg_Cntrl <= register_HOLD;
          Stk_Cntrl <= stack_HOLD;
          PL_En     <= pl_ENABLE;
          Map_En    <= map_DISABLE;
          Vect_En   <= vect_DISABLE;

        end if;  --if ((Cond_Code_En = '0') or (Cond_Code = '1'))

      when uinst_LDCT =>

        Mux_Cntrl <= mux_uPC;
        Reg_Cntrl <= register_LOAD;
        Stk_Cntrl <= stack_HOLD;
        PL_En     <= pl_ENABLE;
        Map_En    <= map_DISABLE;
        Vect_En   <= vect_DISABLE;

      when uinst_LOOP =>

        if ((Cond_Code_En = '0') or (Cond_Code = '1')) then

          Mux_Cntrl <= mux_uPC;
          Reg_Cntrl <= register_HOLD;
          Stk_Cntrl <= stack_POP;
          PL_En     <= pl_ENABLE;
          Map_En    <= map_DISABLE;
          Vect_En   <= vect_DISABLE;

        else

          Mux_Cntrl <= mux_STACK;
          Reg_Cntrl <= register_HOLD;
          Stk_Cntrl <= stack_HOLD;
          PL_En     <= pl_ENABLE;
          Map_En    <= map_DISABLE;
          Vect_En   <= vect_DISABLE;

        end if;  --if ((Cond_Code_En = '0') or (Cond_Code = '1'))

      when uinst_CONT =>

        Mux_Cntrl <= mux_uPC;
        Reg_Cntrl <= register_HOLD;
        Stk_Cntrl <= stack_HOLD;
        PL_En     <= pl_ENABLE;
        Map_En    <= map_DISABLE;
        Vect_En   <= vect_DISABLE;

      when uinst_TWB =>

        if ((Cond_Code_En = '0') or (Cond_Code = '1')) then

          if RegZero = REGeqZERO then

            Mux_Cntrl <= mux_uPC;
            Reg_Cntrl <= register_HOLD;
            Stk_Cntrl <= stack_POP;
            PL_En     <= pl_ENABLE;
            Map_En    <= map_DISABLE;
            Vect_En   <= vect_DISABLE;

          else

            Mux_Cntrl <= mux_uPC;
            Reg_Cntrl <= register_DECR;
            Stk_Cntrl <= stack_POP;
            PL_En     <= pl_ENABLE;
            Map_En    <= map_DISABLE;
            Vect_En   <= vect_DISABLE;

          end if;  -- if RegZero = REGeqZERO

        else

          if RegZero = REGeqZERO then

            Mux_Cntrl <= mux_DIRECT;
            Reg_Cntrl <= register_HOLD;
            Stk_Cntrl <= stack_POP;
            PL_En     <= pl_ENABLE;
            Map_En    <= map_DISABLE;
            Vect_En   <= vect_DISABLE;

          else

            Mux_Cntrl <= mux_STACK;
            Reg_Cntrl <= register_DECR;
            Stk_Cntrl <= stack_HOLD;
            PL_En     <= pl_ENABLE;
            Map_En    <= map_DISABLE;
            Vect_En   <= vect_DISABLE;

          end if;  -- if RegZero = REGeqZERO

        end if;  --if ((Cond_Code_En = '0') or (Cond_Code = '1'))

      when others => null;

    end case;  -- case Instr

  end process U1;

end arch_instruction_pla;
