library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;
use work.am2910_comp.all;

entity multiplexer is
  port (
    D0_in      : in  std_logic_vector(11 downto 0);
    D1_in      : in  std_logic_vector(11 downto 0);
    D2_in      : in  std_logic_vector(11 downto 0);
    D3_in      : in  std_logic_vector(11 downto 0);
    Mux_Sel_in : in  std_logic_vector(1 downto 0);
    Mux_En_in  : in  std_logic;
    Mux_out    : out std_logic_vector(11 downto 0));
end multiplexer;

architecture arch_multiplexer of multiplexer is

  signal D0      : std_logic_vector(11 downto 0);
  signal D1      : std_logic_vector(11 downto 0);
  signal D2      : std_logic_vector(11 downto 0);
  signal D3      : std_logic_vector(11 downto 0);
  signal Mux_Sel : std_logic_vector(1 downto 0);
  signal Mux_En  : std_logic;
  signal Mux     : std_logic_vector(11 downto 0);

begin

  D0      <= D0_in;
  D1      <= D1_in;
  D2      <= D2_in;
  D3      <= D3_in;
  Mux_Sel <= Mux_Sel_in;
  Mux_En  <= Mux_En_in;
  Mux_out <= Mux;

  U1 : process (D0, D1, D2, D3, Mux_Sel, Mux_En)
  begin  -- process U1

    if (Mux_En = '0') then
    
        Mux <= (others => '0');

    else

        case Mux_Sel is

          when "00"   => Mux <= D0;
          when "01"   => Mux <= D1;
          when "10"   => Mux <= D2;
          when "11"   => Mux <= D3;
          when others => Mux <= D0;

        end case;  -- case Mux_Sel

    end if; -- if (Mux_En = '0')

  end process U1;

end arch_multiplexer;
