library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;

use work.am2910_comp.all;

entity stack is

  generic (
    DEPTH : integer := 9);

  port (
    Clock_in  : in  std_logic;
    Stack_in  : in  std_logic_vector(11 downto 0);
    Clr_in    : in  std_logic;
    Push_in   : in  std_logic;
    Pop_in    : in  std_logic;
    Empty_out : out std_logic;
    Full_out  : out std_logic;
    Stack_out : out std_logic_vector(11 downto 0));
end stack;

architecture arch_stack of stack is

  signal Clock   : std_logic;
  signal Clr     : std_logic;
  signal Push    : std_logic;
  signal Pop     : std_logic;
  signal Empty   : std_logic;
  signal Full    : std_logic;
  signal WR_Addr : std_logic_vector(3 downto 0);
  signal RD_Addr : std_logic_vector(3 downto 0);

begin  -- arch_stack

  Clock <= Clock_in;
  Clr   <= Clr_in;
  Push  <= Push_in;
  Pop   <= Pop_in;

  Empty_out <= Empty;
  Full_out  <= Full;

  U1 : stack_pointer
    generic map (
      DEPTH       => DEPTH)
    port map (
      Clock_in    => Clock,
      Clr_in      => Clr,
      Push_in     => Push,
      Pop_in      => Pop,
      Empty_out   => Empty,
      Full_out    => Full,
      WR_Addr_out => WR_Addr,
      RD_Addr_out => RD_Addr);

  U2 : stack_ram
    port map (
      WR_Clk_in   => Clock,
      WR_Addr_in  => WR_Addr,
      RD_Addr_in  => RD_Addr,
      Stack_WE_in => Push,
      WR_Data_in  => Stack_in,
      RD_Data_out => Stack_out);

end arch_stack;
