library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;

use work.am2910_comp.all;

entity stack_pointer is

  generic (
    DEPTH       :     integer := 9);
  port (
    Clock_in    : in  std_logic;
    Clr_in      : in  std_logic;
    Push_in     : in  std_logic;
    Pop_in      : in  std_logic;
    Empty_out   : out std_logic;
    Full_out    : out std_logic;
    WR_Addr_out : out std_logic_vector(3 downto 0);
    RD_Addr_out : out std_logic_vector(3 downto 0));
end stack_pointer;

architecture arch_stack_pointer of stack_pointer is

  signal Clock   : std_logic;
  signal Clr     : std_logic;
  signal Push    : std_logic;
  signal Pop     : std_logic;
  signal Empty   : std_logic;
  signal Full    : std_logic;
  signal WR_Addr : std_logic_vector(3 downto 0);
  signal RD_Addr : std_logic_vector(3 downto 0);

begin  -- arch_stack_pointer

  Clock <= Clock_in;
  Clr   <= Clr_in;
  Push  <= Push_in;
  Pop   <= Pop_in;

  Empty_out   <= Empty;
  Full_out    <= Full;
  WR_Addr_out <= WR_Addr;
  RD_Addr_out <= RD_Addr;

  Empty_Proc : process (Clock)
  begin  -- process Empty_Proc
    if Clock'event and Clock = '1' then  -- rising clock edge
      if Clr = '1' then

        Empty <= '1';

      elsif Pop = '1' and Push = '0' then
        if RD_Addr = "0000" then

          Empty <= '1';

        else

          Empty <= '0';

        end if;  -- if RD_Addr = "0000"

      elsif Pop = '0' and Push = '1' then

        Empty <= '0';

      end if;  -- if Clr = '1'

    end if;  -- if Clock'event and Clock = '1'

  end process Empty_Proc;

  full_process : process (Clock)
  begin  -- process full_process

    if Clock'event and Clock = '1' then  -- rising clock edge
      if clr = '1' then

        full <= '0';

      elsif Pop = '0' and Push = '1' then
        if WR_Addr = (DEPTH - 1) then

          full <= '1';

        else

          full <= '0';

        end if;  -- if WR_Addr = (DEPTH - 1)

      elsif Pop = '1' and Push = '0' then

        full <= '0';

      end if;  -- if clr = '1'
    end if;  -- if Clock'event and Clock = '1'

  end process full_process;

  sp_process : process (Clock)
  begin  -- process sp_process

    if Clock'event and Clock = '1' then  -- rising clock edge
      if clr = '1' then

        WR_Addr <= (others => '0');
        RD_Addr <= (others => '0');

      elsif Pop = '1' and Push = '0' then
        if WR_Addr = "0001" and RD_Addr = "0000" then

          WR_Addr <= WR_Addr - 1;
          RD_Addr <= RD_Addr;

        elsif WR_Addr = "0000" and RD_Addr = "0000" then

          WR_Addr <= WR_Addr;
          RD_Addr <= RD_Addr;

        elsif WR_Addr = (DEPTH - 1) and RD_Addr = (DEPTH - 1) then

          WR_Addr <= WR_Addr;
          RD_Addr <= RD_Addr - 1;

        else

          WR_Addr <= WR_Addr - 1;
          RD_Addr <= RD_Addr - 1;

        end if;  -- if WR_Addr = "0001" and RD_Addr = "0000"

      elsif Push = '1' and Pop = '0' then
        if WR_Addr = "0000" and RD_Addr = "0000" then

          WR_Addr <= WR_Addr + 1;
          RD_Addr <= RD_Addr;

        elsif WR_Addr = (DEPTH - 1) and RD_Addr = (DEPTH - 2) then

          WR_Addr <= WR_Addr;
          RD_Addr <= conv_std_logic_vector((DEPTH - 1), 4);

        elsif WR_Addr = (DEPTH - 1) and RD_Addr = (DEPTH - 1) then

          WR_Addr <= WR_Addr;
          RD_Addr <= RD_Addr;

        else

          WR_Addr <= WR_Addr + 1;
          RD_Addr <= RD_Addr + 1;

        end if;  -- if WR_Addr = "0000" and RD_Addr = "0000"

      else

        WR_Addr <= WR_Addr;
        RD_Addr <= RD_Addr;

      end if;  -- if clr = '1'
    end if;  -- if Clock'event and Clock = '1'

  end process sp_process;

end arch_stack_pointer;
