library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;

use work.User_App_comp.all;

entity Clock_Gen is

  port (
    Clock_in       : in  std_logic;
    Reset_in       : in  std_logic;
    Int_En_out     : out std_logic;
    Display_En_out : out std_logic);
end Clock_Gen;

architecture arch_Clock_Gen of Clock_Gen is

  signal Clock      : std_logic;
  signal Reset      : std_logic;
  signal Int_En     : std_logic;
  signal Display_En : std_logic;

  signal Divider1 : std_logic_vector(4 downto 0);
  signal Divider2 : std_logic_vector(11 downto 0);
  signal Divider3 : std_logic_vector(9 downto 0);

  constant Divider1_Term : std_logic_vector(4 downto 0)  := "10111";
  constant Divider2_Term : std_logic_vector(11 downto 0) := "100111000010";
  constant Divider3_Term : std_logic_vector(8 downto 0)  := "110001110";

  signal Divider1_TC : std_logic;
  signal Divider2_TC : std_logic;
  signal Divider3_TC : std_logic;

begin  -- arch_Clock_Gen

  Clock <= Clock_in;
  Reset <= Reset_in;

  Int_En_out     <= Int_En;
  Display_En_out <= Display_En;

  U1a : process (Clock, Reset)
  begin
    if Reset = '1' then

      Divider1 <= (others => '0');

    elsif Clock'event and Clock = '1' then
      if Divider1_TC = '1' then

        Divider1 <= (others => '0');

      else

        Divider1 <= Divider1 + 1;

      end if;  -- if Divider1_TC = '1'
    end if;  -- if Reset = '1'

  end process U1a;

  U1b : process (Clock, Reset)
  begin
    if Reset = '1' then

      Divider1_TC <= '0';

    elsif Clock'event and Clock = '1' then
      if Divider1 = Divider1_Term then

        Divider1_TC <= '1';

      else

        Divider1_TC <= '0';

      end if;  -- if Divider1 = Divider1_Term
    end if;  -- if Reset = '1'

  end process U1b;

  U2a : process (Clock, Reset)
  begin
    if Reset = '1' then

      Divider2 <= (others => '0');

    elsif Clock'event and Clock = '1' then
      if Divider1_TC = '1' then
        if Divider2_TC = '1' then

          Divider2 <= (others => '0');

        else

          Divider2 <= Divider2 + 1;

        end if;  -- if Divider2_TC = '1'
      end if;  -- if Divider1_TC = '1'
    end if;  -- if Reset = '1'

  end process U2a;

  U2b : process (Clock, Reset)
  begin
    if Reset = '1' then

      Divider2_TC <= '0';

    elsif Clock'event and Clock = '1' then
      if Divider1_TC = '1' then
        if Divider2 = Divider2_Term then

          Divider2_TC <= '1';

        else

          Divider2_TC <= '0';

        end if;  -- if Divider2 = Divider2_Term 
      end if;  -- if Divider1_TC = '1'
    end if;  -- if Reset = '1'

  end process U2b;

  U3a : process (Clock, Reset)
  begin
    if Reset = '1' then

      Divider3 <= (others => '0');

    elsif Clock'event and Clock = '1' then
      if Divider1_TC = '1' and Divider2_TC = '1' then
        if Divider3_TC = '1' then

          Divider3 <= (others => '0');

        else

          Divider3 <= Divider3 + 1;

        end if;  -- if Divider3_TC = '1'
      end if;  -- if Divider1_TC = '1' and Divider2_TC = '1'
    end if;  -- if Reset = '1'

  end process U3a;

  U3b : process (Clock, Reset)
  begin
    if Reset = '1' then

      Divider3_TC <= '0';

    elsif Clock'event and Clock = '1' then
      if Divider1_TC = '1' and Divider2_TC = '1' then
        if Divider3 = Divider3_Term then

          Divider3_TC <= '1';

        else

          Divider3_TC <= '0';

        end if;  -- if Divider3 = Divider3_Term
      end if;  -- if Divider1_TC = '1' and Divider2_TC = '1'
    end if;  -- if Reset = '1'

  end process U3b;

  U4 : process (Clock, Reset)
  begin
    if Reset = '1' then

      Display_En <= '0';

    elsif Clock'event and Clock = '1' then
      if Divider1_TC = '1' and Divider2_TC = '1' then

        Display_En <= '1';

      else

        Display_En <= '0';

      end if;  -- if Divider1_TC = '1' and Divider2_TC = '1'
    end if;  -- if Reset = '1'

  end process U4;

  U5 : process (Clock, Reset)
  begin
    if Reset = '1' then

      Int_En <= '0';

    elsif Clock'event and Clock = '1' then
      if Divider1_TC = '1' and Divider2_TC = '1' and Divider3_TC = '1' then

        Int_En <= '1';

      else

        Int_En <= '0';

      end if;  -- if Divider1_TC = '1' and Divider2_TC = '1' and Divider3_TC = '1'
    end if;  -- if Reset = '1'

  end process U5;

end arch_Clock_Gen;
