library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;

use work.User_App_comp.all;

entity Display_Cntrl is

  port (
    Clock_in       : in  std_logic;
    Reset_in       : in  std_logic;
    Enable_in      : in  std_logic;
    DI_in          : in  std_logic_vector(15 downto 0);
    Segment_A_out  : out std_logic;
    Segment_B_out  : out std_logic;
    Segment_C_out  : out std_logic;
    Segment_D_out  : out std_logic;
    Segment_E_out  : out std_logic;
    Segment_F_out  : out std_logic;
    Segment_G_out  : out std_logic;
    Segment_DP_out : out std_logic;
    Segment_AN_out : out std_logic_vector(3 downto 0));
end Display_Cntrl;

architecture arch_Display_Cntrl of Display_Cntrl is

  signal Clock      : std_logic;
  signal Reset      : std_logic;
  signal Enable     : std_logic;
  signal DI         : std_logic_vector(15 downto 0);
  signal Segment_A  : std_logic;
  signal Segment_B  : std_logic;
  signal Segment_C  : std_logic;
  signal Segment_D  : std_logic;
  signal Segment_E  : std_logic;
  signal Segment_F  : std_logic;
  signal Segment_G  : std_logic;
  signal Segment_DP : std_logic;
  signal Segment_AN : std_logic_vector(3 downto 0);

  signal Hex_Value     : std_logic_vector(3 downto 0);
  signal Segment_Value : std_logic_vector(6 downto 0);

  signal Divider : std_logic_vector(6 downto 0);
  constant Divider_Term : std_logic_vector(6 downto 0)  := "1100010";
  signal Divider_TC : std_logic;

begin  -- arch_Display_Cntrl

  Clock  <= Clock_in;
  Reset  <= Reset_in;
  Enable <= Enable_in;
  DI     <= DI_in;

  Segment_A_out  <= Segment_A;
  Segment_B_out  <= Segment_B;
  Segment_C_out  <= Segment_C;
  Segment_D_out  <= Segment_D;
  Segment_E_out  <= Segment_E;
  Segment_F_out  <= Segment_F;
  Segment_G_out  <= Segment_G;
  Segment_DP_out <= Segment_DP;
  Segment_AN_out <= Segment_AN;

  Segment_A <= Segment_Value(6);
  Segment_B <= Segment_Value(5);
  Segment_C <= Segment_Value(4);
  Segment_D <= Segment_Value(3);
  Segment_E <= Segment_Value(2);
  Segment_F <= Segment_Value(1);
  Segment_G <= Segment_Value(0);

  U1 : process (Clock, Reset)
  begin
    if Reset = '1' then

      Segment_AN <= "0111";

    elsif Clock'event and Clock = '1' then
      if Enable = '1' then

        Segment_AN(3) <= Segment_AN(2);
        Segment_AN(2) <= Segment_AN(1);
        Segment_AN(1) <= Segment_AN(0);
        Segment_AN(0) <= Segment_AN(3);

      else

        Segment_AN <= Segment_AN;

      end if;  -- if Enable = '1'
    end if;  -- if Reset = '1'

  end process U1;

  U2 : process (DI, Segment_AN)
  begin

    case Segment_AN is

      when "1110" => Hex_Value <= DI(3 downto 0);

      when "1101" => Hex_Value <= DI(7 downto 4);

      when "1011" => Hex_Value <= DI(11 downto 8);

      when "0111" => Hex_Value <= DI(15 downto 12);

      when others => Hex_Value <= (others => '0');

    end case;

  end process U2;

  U3 : process (Hex_Value)
  begin

    case Hex_Value is

      when X"0"   => Segment_Value <= "0000001";
      when X"1"   => Segment_Value <= "1001111";
      when X"2"   => Segment_Value <= "0010010";
      when X"3"   => Segment_Value <= "0000110";
      when X"4"   => Segment_Value <= "1001100";
      when X"5"   => Segment_Value <= "0100100";
      when X"6"   => Segment_Value <= "0100000";
      when X"7"   => Segment_Value <= "0001111";
      when X"8"   => Segment_Value <= "0000000";
      when X"9"   => Segment_Value <= "0000100";
      when X"A"   => Segment_Value <= "0001000";
      when X"B"   => Segment_Value <= "1100000";
      when X"C"   => Segment_Value <= "0110001";
      when X"D"   => Segment_Value <= "1000010";
      when X"E"   => Segment_Value <= "0110000";
      when X"F"   => Segment_Value <= "0111000";
      when others => null;

    end case;

  end process U3;

  U4a : process (Clock, Reset)
  begin
    if Reset = '1' then

      Divider <= (others => '0');

    elsif Clock'event and Clock = '1' then
      if Enable = '1' then
        if Divider_TC = '1' then

          Divider <= (others => '0');

        else

          Divider <= Divider + 1;

        end if;  -- if Divider_TC = '1'
      end if;  -- if Enable = '1'
    end if;  -- if Reset = '1'

  end process U4a;

  U4b : process (Clock, Reset)
  begin
    if Reset = '1' then

      Divider_TC <= '0';

    elsif Clock'event and Clock = '1' then
      if Enable = '1' then
        if Divider = Divider_Term then

          Divider_TC <= '1';

        else

          Divider_TC <= '0';

        end if;  -- if Divider = Divider_Term
      end if;  -- if Enable = '1'
    end if;  -- if Reset = '1'

  end process U4b;

  U5 : process (Clock, Reset)
  begin
    if Reset = '1' then

      Segment_DP <= '0';

    elsif Clock'event and Clock = '1' then
      if Divider_TC = '1' then

        Segment_DP <= not Segment_DP;

      end if;  -- if Enable = '1'
    end if;  -- if Reset = '0'

  end process U5;

end arch_Display_Cntrl;
