library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

use work.smp_const.all;

entity IR_Map_Mux is

  port (
    Clock_in             : in  std_logic;
    Reset_in             : in  std_logic;
    IR_in                : in  std_logic_vector(15 downto 0);
    IR_Map_Mux_Select_in : in  std_logic_vector(1 downto 0);
    IR_Map_out           : out std_logic_vector(11 downto 0));

end IR_Map_Mux;

architecture arch_IR_Map_Mux of IR_Map_Mux is

  signal Clock             : std_logic;
  signal Reset             : std_logic;
  signal IR                : std_logic_vector(15 downto 0);
  signal IR_Map_Mux_Select : std_logic_vector(1 downto 0);
  signal IR_Map            : std_logic_vector(11 downto 0);

  type ROM1_type is array (0 to 7) of std_logic_vector(11 downto 0);
  type ROM2_type is array (0 to 31) of std_logic_vector(11 downto 0);

  signal LUT0 : ROM1_type :=
    (
      pre_Reg_Reg, pre_Reg_Ind, pre_Reg_Imm, pre_Reg_1op,
      pre_Ind_Reg, pre_Ind_Ind, pre_Ind_Imm, pre_Ind_1op);

  signal LUT1 : ROM2_type :=
    (
      inst_LD, inst_ADD, inst_ADDC, inst_INC,
      inst_SUB, inst_SUBC, inst_DEC, inst_AND,
      inst_OR, inst_XOR, inst_CMP, inst_SRL,
      inst_SLL, inst_SRA, inst_SLA, inst_IN,
      inst_OUT, inst_LDSP, inst_LDIV, inst_STSP,
      inst_DI, inst_EI, inst_JMP, inst_CJMP,
      inst_CALL, inst_RET, inst_RETI, inst_NOP,
      inst_NOP, inst_NOP, inst_NOP, inst_NOP);

  signal LUT2 : ROM1_type :=
    (
      post_Reg_Reg, post_Reg_Ind, post_Reg_Imm, post_Reg_1op,
      post_Ind_Reg, post_Ind_Ind, post_Ind_Imm, post_Ind_1op);

begin  -- arch_IR_Map_Mux

  Clock             <= Clock_in;
  Reset             <= Reset_in;
  IR                <= IR_in;
  IR_Map_Mux_Select <= IR_Map_Mux_Select_in;

  IR_Map <= LUT0(conv_integer(IR(10 downto 8)))  when IR_Map_Mux_Select = "00" else
            LUT1(conv_integer(IR(15 downto 11))) when IR_Map_Mux_Select = "01" else
            LUT2(conv_integer(IR(10 downto 8)))  when IR_Map_Mux_Select = "10" else
            (others => '0');

  U1 : process (Clock, Reset)
  begin  -- process Clock
    if Reset = '1' then                 -- asynchronous reset (active high)

      IR_Map_out <= (others => '0');

    elsif Clock'event and Clock = '1' then  -- rising clock edge

      IR_Map_out <= IR_Map;

    end if;  -- if Reset = '1'

  end process U1;

end arch_IR_Map_Mux;
