library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity Stat_Reg is

  port (
    Clock_in       : in  std_logic;
    Reset_in       : in  std_logic;
    Zero_in        : in  std_logic;
    Carry_in       : in  std_logic;
    Sign_in        : in  std_logic;
    Overflow_in    : in  std_logic;
    D_in           : in  std_logic_vector(3 downto 0);
    Shift_Carry_in : in  std_logic;
    SR_Load_in     : in  std_logic;
    SRC_Load_in    : in  std_logic;
    SR_out         : out std_logic_vector(3 downto 0));

end Stat_Reg;

architecture arch_Stat_Reg of Stat_Reg is

  signal Clock       : std_logic;
  signal Reset       : std_logic;
  signal Zero        : std_logic;
  signal Carry       : std_logic;
  signal Sign        : std_logic;
  signal Overflow    : std_logic;
  signal D           : std_logic_vector(3 downto 0);
  signal Shift_Carry : std_logic;
  signal SR_Load     : std_logic;
  signal SRC_Load    : std_logic;
  signal SR          : std_logic_vector(3 downto 0);

begin  -- arch_Stat_Reg

  Clock       <= Clock_in;
  Reset       <= Reset_in;
  Zero        <= Zero_in;
  Carry       <= Carry_in;
  Sign        <= Sign_in;
  Overflow    <= Overflow_in;
  D           <= D_in;
  Shift_Carry <= Shift_Carry_in;
  SR_Load     <= SR_Load_in;
  SRC_Load    <= SRC_Load_in;

  SR_out <= SR;

  U1 : process (Clock, Reset)
  begin  -- process U1
    if Reset = '1' then                 -- asynchronous reset (active high)

      SR <= (others => '0');

    elsif Clock'event and Clock = '1' then  -- rising clock edge
      if SR_Load = '1' and SRC_Load = '0' then

        SR(0) <= Zero;
        SR(1) <= Carry;
        SR(2) <= Sign;
        SR(3) <= Overflow;

      elsif SR_Load = '0' and SRC_Load = '1' then

        SR(0) <= SR(0);
        SR(1) <= Shift_Carry;
        SR(2) <= SR(2);
        SR(3) <= SR(3);

      elsif SR_Load = '1' and SRC_Load = '1' then

        SR <= D;

      else

        SR <= SR;

      end if;  -- if SR_Load = '1' and SRC_Load = '0'
    end if;  -- if Reset = '1'

  end process U1;


--   U1 : process (Clock, Reset)
--   begin  -- process U1
--     if Reset = '1' then                 -- asynchronous reset (active high)

--       SR(0) <= '0';
--       SR(2) <= '0';
--       SR(3) <= '0';

--     elsif Clock'event and Clock = '1' then  -- rising clock edge
--       if SR_Load = '1' then

--         SR(0) <= Zero;
--         SR(2) <= Sign;
--         SR(3) <= Overflow;

--       end if;  -- if SR_Load = '1'
--     end if;  -- if Reset = '1'

--   end process U1;

--   U2 : process (Clock, Reset)
--   begin  -- process U2
--     if Reset = '1' then                 -- asynchronous reset (active high

--       SR(1) <= '0';

--     elsif Clock'event and Clock = '1' then  -- rising clock edge
--       if SR_Load = '1' then

--         SR(1) <= Carry;

--       elsif SRC_Load = '1' then

--         SR(1) <= Shift_Carry;

--       end if;  -- if SR_Load = '1'

--     end if;  -- if Reset = '1'

--   end process U2;

end arch_Stat_Reg;
