library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

use work.smp_comp.all;

entity smp is

  port (
    CP_in         : in  std_logic;
    Reset_in      : in  std_logic;
    Int_in        : in  std_logic;
    MDI_in        : in  std_logic_vector(15 downto 0);
    IODI_in       : in  std_logic_vector(15 downto 0);
    MWait_in      : in  std_logic;
    IOWait_in     : in  std_logic;
-------------------------------------------------------------------------------
    uProgAddr_out : out std_logic_vector(11 downto 0);
    PState_out    : out std_logic_vector(5 downto 0);
    MAR_load_out  : out std_logic;
    MDOR_load_out : out std_logic;
    -------------------------------------------------------------------------------
    MAddr_out     : out std_logic_vector(15 downto 0);
    MDO_out       : out std_logic_vector(15 downto 0);
    MRD_out       : out std_logic;
    MWR_out       : out std_logic;
    IOAddr_out    : out std_logic_vector(15 downto 0);
    IODO_out      : out std_logic_vector(15 downto 0);
    IORD_out      : out std_logic;
    IOWR_out      : out std_logic);

end smp;

architecture arch_smp of smp is

  signal CP     : std_logic;
  signal Reset  : std_logic;
  signal Int    : std_logic;
  signal MDI    : std_logic_vector(15 downto 0);
  signal MDIR   : std_logic_vector(15 downto 0);
  signal IODI   : std_logic_vector(15 downto 0);
  signal IODIR  : std_logic_vector(15 downto 0);
  signal MWait  : std_logic;
  signal IOWait : std_logic;

  signal MAR   : std_logic_vector(15 downto 0);
  signal MDOR  : std_logic_vector(15 downto 0);
  signal IOAR  : std_logic_vector(15 downto 0);
  signal IODOR : std_logic_vector(15 downto 0);

  signal am2910_D     : std_logic_vector(11 downto 0);
  signal am2910_nPL   : std_logic;
  signal am2910_nMAP  : std_logic;
  signal am2910_nVECT : std_logic;
  signal am2910_CC    : std_logic;
  signal am2910_nCC   : std_logic;
  signal uProgAddr    : std_logic_vector(11 downto 0);

  signal am2901_A        : std_logic_vector(3 downto 0);
  signal am2901_B        : std_logic_vector(3 downto 0);
  signal am2901_CI       : std_logic;
  signal am2901_D        : std_logic_vector(15 downto 0);
  signal am2901_Y        : std_logic_vector(15 downto 0);
  signal am2901_Feq0     : std_logic;
  signal am2901_F15      : std_logic;
  signal am2901_CO       : std_logic;
  signal am2901_OVR      : std_logic;
  signal am2901_Q0in     : std_logic;
  signal am2901_Q15in    : std_logic;
  signal am2901_Q0out    : std_logic;
  signal am2901_Q15out   : std_logic;
  signal am2901_RAM0in   : std_logic;
  signal am2901_RAM15in  : std_logic;
  signal am2901_RAM0out  : std_logic;
  signal am2901_RAM15out : std_logic;

  signal IR_Test_Pol    : std_logic;
  signal IR_Test_Select : std_logic_vector(2 downto 0);
  signal IR_am2910_D    : std_logic_vector(11 downto 0);
  signal IR_am2901_A    : std_logic_vector(3 downto 0);
  signal IR_am2901_B    : std_logic_vector(3 downto 0);
-- signal IR_am2901_CI : std_logic;

  signal Test_Pol    : std_logic;
  signal Test_Select : std_logic_vector(2 downto 0);

  signal Shift_Carry      : std_logic;
  signal Shift_Carry_Load : std_logic;

  signal IR : std_logic_vector(15 downto 0);
  signal SR : std_logic_vector(3 downto 0);
  signal IE : std_logic;
-- signal Int_Reg : std_logic;

  signal Status_Reg : std_logic_vector(15 downto 0);

--Modify the following lines for your application  ------------------------------

  constant CONTROL_STORE_DEPTH : integer := 512;
  constant MICROWORD_WIDTH     : integer := 72;

-------------------------------------------------------------------------------

  signal Pipeline : std_logic_vector((MICROWORD_WIDTH - 1) downto 0);

  signal CS_DI : std_logic_vector((MICROWORD_WIDTH - 1) downto 0);

--Modify the following lines for your application  ------------------------------

  alias am2910_I    : std_logic_vector(3 downto 0) is Pipeline(71 downto 68);
  alias CS_am2910_D : std_logic_vector(11 downto 0) is Pipeline(63 downto 52);

  alias CS_Test_Pol    : std_logic is Pipeline(51);
  alias CS_Test_Select : std_logic_vector(2 downto 0) is Pipeline(50 downto 48);

  alias am2901_I     : std_logic_vector(8 downto 0) is Pipeline(47 downto 39);
  alias CS_am2901_A  : std_logic_vector(3 downto 0) is Pipeline(38 downto 35);
  alias CS_am2901_B  : std_logic_vector(3 downto 0) is Pipeline(34 downto 31);
  alias CS_am2901_Ci : std_logic is Pipeline(30);
  alias CS_am2901_D  : std_logic_vector(15 downto 0) is Pipeline(67 downto 52);

  alias MDIR_Load  : std_logic is Pipeline(29);
  alias MDOR_Load  : std_logic is Pipeline(28);
  alias MAR_Load   : std_logic is Pipeline(27);
  alias IODIR_Load : std_logic is Pipeline(26);
  alias IODOR_Load : std_logic is Pipeline(25);
  alias IOAR_Load  : std_logic is Pipeline(24);
  alias IR_Load    : std_logic is Pipeline(23);
  alias SR_Load    : std_logic is Pipeline(22);
  alias IE_Load    : std_logic is Pipeline(21);

  alias CI_Mux_Select          : std_logic is Pipeline(20);
  alias A_Mux_Select           : std_logic is Pipeline(19);
  alias B_Mux_Select           : std_logic is Pipeline(18);
  alias Test_Source_Mux_Select : std_logic is Pipeline(17);
  alias am2901_D_Mux_Select    : std_logic_vector(1 downto 0) is Pipeline(16 downto 15);

  alias MRD  : std_logic is Pipeline(14);
  alias MWR  : std_logic is Pipeline(13);
  alias IORD : std_logic is Pipeline(12);
  alias IOWR : std_logic is Pipeline(11);

  alias Shift_Select : std_logic_vector(2 downto 0) is Pipeline(10 downto 8);

  alias IR_Map_Mux_Select : std_logic_vector(1 downto 0) is Pipeline(7 downto 6);

  alias PState : std_logic_vector(5 downto 0) is Pipeline(5 downto 0);

-------------------------------------------------------------------------------

begin  -- arch_smp

-------------------------------------------------------------------------------

  CP <= CP_in;
  Reset  <= Reset_in;
  Int    <= Int_in and IE;
  MDI    <= MDI_in;
  IODI   <= IODI_in;
  MWait  <= MWait_in;
  IOWait <= IOWait_in;

  MAddr_out <= MAR;
  MDO_out   <= MDOR;
  MRD_out   <= MRD;
  MWR_out   <= MWR;

  IOAddr_out <= IOAR;
  IODO_out   <= IODOR;
  IORD_out   <= IORD;
  IOWR_out   <= IOWR;

  Status_Reg(3 downto 0)  <= SR(3 downto 0);
  Status_Reg(15)          <= IE;
  Status_Reg(14 downto 4) <= (others => '0');

-------------------------------------------------------------------------------
  Sync_Signals : process (CP, Reset)
  begin  -- process Sync_Signals
    if Reset = '1' then                 -- asynchronous reset (active high)

      uProgAddr_out <= (others => '0');
      PState_out    <= (others => '0');
      MAR_load_out <= '0';
      MDOR_load_out <= '0';
      
    elsif CP'event and CP = '1' then    -- rising clock edge

      uProgAddr_out <= uProgAddr;
      PState_out    <= Pstate;
      MAR_load_out  <= MAR_load;
      MDOR_load_out <= MDOR_load;

    end if;

  end process Sync_Signals;
-----------------------------------------------------------------------------

  IR_Test_Select(1 downto 0) <= IR(7 downto 6);
  IR_Test_Select(2)          <= '0';
  IR_Test_Pol                <= '0';

  IR_am2901_A <= '0' & IR(2 downto 0);
  IR_am2901_B <= '0' & IR(5 downto 3);

  CS_DI <= (others => '0');

  U1a : mux_2to1
    generic map (
      WIDTH         => 3)
    port map (
      D0_in         => CS_Test_Select,
      D1_in         => IR_Test_Select,
      Mux_Select_in => Test_Source_Mux_Select,
      Mux_out       => Test_Select);

  U1b : mux_2to1b
    port map (
      D0_in         => CS_Test_Pol,
      D1_in         => IR_Test_Pol,
      Mux_Select_in => Test_Source_Mux_Select,
      Mux_out       => Test_Pol);

  U2 : IR_Map_Mux
    port map (
      Clock_in             => CP,
      Reset_in             => Reset,
      IR_in                => IR,
      IR_Map_Mux_Select_in => IR_Map_Mux_Select,
      IR_Map_out           => IR_am2910_D);

  U3 : mux_2to1
    generic map (
      WIDTH         => 12)
    port map (
      D0_in         => IR_am2910_D,
      D1_in         => CS_am2910_D,
      Mux_Select_in => am2910_nMAP,
      Mux_out       => am2910_D);

  U4 : mux_8to1b
    port map (
      D0_in         => SR(0),
      D1_in         => SR(1),
      D2_in         => SR(2),
      D3_in         => SR(3),
      D4_in         => Int,
      D5_in         => MWait,
      D6_in         => IOWait,
      D7_in         => '1',
      Mux_Select_in => Test_Select,
      Mux_out       => am2910_CC);

  am2910_nCC <= not (am2910_CC xor Test_Pol);

  U5 : Am2910
    generic map (
      STACK_DEPTH  => 9)
    port map (
      CP_in        => CP,
      nCC_in       => am2910_nCC,
      nCCEN_in     => '0',
      CI_in        => '1',
      D_in         => am2910_D,
      I_in         => am2910_I,
      nOE_in       => '0',
      nRLD_in      => '1',
      nMap_En_out  => am2910_nMAP,
      nPL_En_out   => am2910_nPL,
      nVect_En_out => am2910_nVECT,
      Y_out        => uProgAddr,
      nFull_out    => open);

  U6 : Control_Store
    generic map (
      WIDTH    => MICROWORD_WIDTH,
      DEPTH    => CONTROL_STORE_DEPTH)
    port map (
      CP_in    => CP,
      Reset_in => Reset,
      WE_in    => '0',
      EN_in    => '1',
      Addr_in  => uProgAddr,
      DI_in    => CS_DI,
      DO_out   => Pipeline);

  U7 : mux_2to1
    generic map (
      WIDTH         => 4)
    port map (
      D0_in         => CS_am2901_A,
      D1_in         => IR_am2901_A,
      Mux_Select_in => A_Mux_Select,
      Mux_out       => am2901_A);

  U8 : mux_2to1
    generic map (
      WIDTh         => 4)
    port map (
      D0_in         => CS_am2901_B,
      D1_in         => IR_am2901_B,
      Mux_Select_in => B_Mux_Select,
      Mux_out       => am2901_B);

  U9 : mux_2to1b
    port map (
      D0_in         => CS_am2901_CI,
      D1_in         => SR(1),
      Mux_Select_in => CI_Mux_Select,
      Mux_out       => am2901_CI);

  U10 : mux_4to1
    generic map (
      WIDTH         => 16)
    port map (
      D0_in         => CS_am2901_D,
      D1_in         => MDIR,
      D2_in         => IODIR,
      D3_in         => Status_Reg,
      Mux_Select_in => am2901_D_Mux_Select,
      Mux_out       => am2901_D);

  U11 : am2901
    generic map (
      SLICE_WIDTH     => 16)
    port map (
      CP_in           => CP,
      A_in            => am2901_A,
      B_in            => am2901_B,
      D_in            => am2901_D,
      I_in            => am2901_I,
      nOE_in          => '0',
      CI_in           => am2901_CI,
      Q_LSB_in        => am2901_Q0in,
      Q_MSB_in        => am2901_Q15in,
      RAM_LSB_in      => am2901_RAM0in,
      RAM_MSB_in      => am2901_RAM15in,
      Y_out           => am2901_Y,
      nG_out          => open,
      nP_out          => open,
      Feq0_out        => am2901_Feq0,
      F_MSB_out       => am2901_F15,
      OVR_out         => am2901_OVR,
      CO_out          => am2901_CO,
      Q_LSB_out       => am2901_Q0out,
      Q_MSB_out       => am2901_Q15out,
      Q_LSB_Dir_out   => open,
      Q_MSB_Dir_out   => open,
      RAM_LSB_out     => am2901_RAM0out,
      RAM_MSB_out     => am2901_RAM15out,
      RAM_LSB_Dir_out => open,
      RAM_MSB_Dir_out => open);

  U12 : Shift_Cntrl
    port map (
      Q_LSB_in       => am2901_Q0out,
      Q_MSB_in       => am2901_Q15out,
      RAM_LSB_in     => am2901_RAM0out,
      RAM_MSB_in     => am2901_RAM15out,
      Carry_in       => SR(1),
      I_in           => Shift_Select,
      Q_LSB_out      => am2901_Q0in,
      Q_MSB_out      => am2901_Q15in,
      RAM_LSB_out    => am2901_RAM0in,
      RAM_MSB_out    => am2901_RAM15in,
      Carry_out      => Shift_Carry,
      Carry_Load_out => Shift_Carry_Load);

  U13 : Stat_Reg
    port map (
      Clock_in       => CP,
      Reset_in       => Reset,
      Zero_in        => am2901_Feq0,
      Carry_in       => am2901_CO,
      Sign_in        => am2901_F15,
      Overflow_in    => am2901_OVR,
      D_in           => am2901_Y(3 downto 0),
      Shift_Carry_in => Shift_Carry,
      SR_Load_in     => SR_Load,
      SRC_Load_in    => Shift_Carry_Load,
      SR_out         => SR);

  U14 : reg_1be
    port map (
      Clock_in  => CP,
      Reset_in  => Reset,
      Enable_in => IE_Load,
      D_in      => am2901_Y(15),
      Q_out     => IE);

  U15 : reg_vbe
    generic map (
      WIDTH     => 16)
    port map (
      Clock_in  => CP,
      Reset_in  => Reset,
      Enable_in => IR_Load,
      D_in      => MDIR,
      Q_out     => IR);

  U16 : reg_vbe
    generic map (
      WIDTH     => 16)
    port map (
      Clock_in  => CP,
      Reset_in  => Reset,
      Enable_in => MDOR_Load,
      D_in      => am2901_Y,
      Q_out     => MDOR);

  U17 : reg_vbe
    generic map (
      WIDTH     => 16)
    port map (
      Clock_in  => CP,
      Reset_in  => Reset,
      Enable_in => MAR_Load,
      D_in      => am2901_Y,
      Q_out     => MAR);

  U18 : reg_vbe
    generic map (
      WIDTH     => 16)
    port map (
      Clock_in  => CP,
      Reset_in  => Reset,
      Enable_in => MDIR_Load,
      D_in      => MDI,
      Q_out     => MDIR);

  U19 : reg_vbe
    generic map (
      WIDTH     => 16)
    port map (
      Clock_in  => CP,
      Reset_in  => Reset,
      Enable_in => IODOR_Load,
      D_in      => am2901_Y,
      Q_out     => IODOR);

  U20 : reg_vbe
    generic map (
      WIDTH     => 16)
    port map (
      Clock_in  => CP,
      Reset_in  => Reset,
      Enable_in => IOAR_Load,
      D_in      => am2901_Y,
      Q_out     => IOAR);

  U21 : reg_vbe
    generic map (
      WIDTH     => 16)
    port map (
      Clock_in  => CP,
      Reset_in  => Reset,
      Enable_in => IODIR_Load,
      D_in      => IODI,
      Q_out     => IODIR);

end arch_smp;
