library ieee;
use ieee.std_logic_1164.all;

package smp_const is

--
-- Shift Control Constants
--
  constant RSHFT  : std_logic_vector(2 downto 0) := "000";
  constant LSHFT  : std_logic_vector(2 downto 0) := "001";
  constant RSHFTA : std_logic_vector(2 downto 0) := "010";
  constant LSHFTA : std_logic_vector(2 downto 0) := "011";
  constant RROT   : std_logic_vector(2 downto 0) := "100";
  constant LROT   : std_logic_vector(2 downto 0) := "101";

--
-- Pre-operation LUT
--
  constant pre_Reg_Reg : std_logic_vector(11 downto 0) := X"020";
  constant pre_Reg_Ind : std_logic_vector(11 downto 0) := X"021";
  constant pre_Reg_Imm : std_logic_vector(11 downto 0) := X"024";
  constant pre_Reg_1op : std_logic_vector(11 downto 0) := X"028";
  constant pre_Ind_Reg : std_logic_vector(11 downto 0) := X"02A";
  constant pre_Ind_Ind : std_logic_vector(11 downto 0) := X"02B";
  constant pre_Ind_Imm : std_logic_vector(11 downto 0) := X"02E";
  constant pre_Ind_1op : std_logic_vector(11 downto 0) := X"034";

--
-- Instruction LUT
--
  constant inst_LD   : std_logic_vector(11 downto 0) := X"03C";
  constant inst_ADD  : std_logic_vector(11 downto 0) := X"03D";
  constant inst_ADDC : std_logic_vector(11 downto 0) := X"03E";
  constant inst_INC  : std_logic_vector(11 downto 0) := X"03F";
  constant inst_SUB  : std_logic_vector(11 downto 0) := X"040";
  constant inst_SUBC : std_logic_vector(11 downto 0) := X"041";
  constant inst_DEC  : std_logic_vector(11 downto 0) := X"042";
  constant inst_AND  : std_logic_vector(11 downto 0) := X"043";
  constant inst_OR   : std_logic_vector(11 downto 0) := X"044";
  constant inst_XOR  : std_logic_vector(11 downto 0) := X"045";
  constant inst_CMP  : std_logic_vector(11 downto 0) := X"046";
  constant inst_SRL  : std_logic_vector(11 downto 0) := X"047";
  constant inst_SLL  : std_logic_vector(11 downto 0) := X"048";
  constant inst_SRA  : std_logic_vector(11 downto 0) := X"049";
  constant inst_SLA  : std_logic_vector(11 downto 0) := X"04A";
  constant inst_IN   : std_logic_vector(11 downto 0) := X"04B";
  constant inst_OUT  : std_logic_vector(11 downto 0) := X"04E";
  constant inst_LDSP : std_logic_vector(11 downto 0) := X"052";
  constant inst_LDIV : std_logic_vector(11 downto 0) := X"053";
  constant inst_STSP : std_logic_vector(11 downto 0) := X"054";
  constant inst_DI   : std_logic_vector(11 downto 0) := X"055";
  constant inst_EI   : std_logic_vector(11 downto 0) := X"056";
  constant inst_JMP  : std_logic_vector(11 downto 0) := X"057";
  constant inst_CJMP : std_logic_vector(11 downto 0) := X"058";
  constant inst_CALL : std_logic_vector(11 downto 0) := X"05A";
  constant inst_RET  : std_logic_vector(11 downto 0) := X"05E";
  constant inst_RETI : std_logic_vector(11 downto 0) := X"061";
  constant inst_NOP  : std_logic_vector(11 downto 0) := X"012";

--
-- Post-operation LUT
--
  constant post_Reg_Reg : std_logic_vector(11 downto 0) := X"037";
  constant post_Reg_Ind : std_logic_vector(11 downto 0) := X"037";
  constant post_Reg_Imm : std_logic_vector(11 downto 0) := X"037";
  constant post_Reg_1op : std_logic_vector(11 downto 0) := X"037";

  constant post_Ind_Reg : std_logic_vector(11 downto 0) := X"038";
  constant post_Ind_Ind : std_logic_vector(11 downto 0) := X"038";
  constant post_Ind_Imm : std_logic_vector(11 downto 0) := X"038";
  constant post_Ind_1op : std_logic_vector(11 downto 0) := X"038";

end smp_const;
